"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const treeTraverse_1 = require("../parser/treeTraverse");
/**
 * Class to help identify parameters and return statements in
 * a kerboscript function
 */
class FunctionScan extends treeTraverse_1.TreeTraverse {
    /**
     * Function scan constructor
     */
    constructor() {
        super();
        this.result = {
            requiredParameters: 0,
            optionalParameters: 0,
            return: false,
        };
    }
    /**
     * Scan the function node for parameters and return statements
     * @param node function body
     */
    scan(node) {
        this.result = {
            requiredParameters: 0,
            optionalParameters: 0,
            return: false,
        };
        this.stmtAction(node);
        return Object.assign({}, this.result);
    }
    /**
     * Indicate that the function has a return statement
     * @param _ return statement
     */
    visitReturn(_) {
        this.result.return = true;
    }
    /**
     * Don't proceed further in if enter another have a function
     * @param _ function declaration
     */
    visitDeclFunction(_) { }
    /**
     * Add required and optional parameters when found
     * @param decl parameter declaration
     */
    visitDeclParameter(decl) {
        this.result.requiredParameters += decl.requiredParameters.length;
        this.result.optionalParameters += decl.optionalParameters.length;
    }
    /**
     * DOn't proceed further if another lambda is encountered
     * @param expr lambda expression
     */
    visitLambda(_) { }
}
exports.FunctionScan = FunctionScan;
//# sourceMappingURL=functionScan.js.map