"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../parser/types");
const vscode_languageserver_1 = require("vscode-languageserver");
const base_1 = require("../parser/base");
const arrayUtils_1 = require("../utilities/arrayUtils");
class Script extends base_1.NodeBase {
    constructor(uri, stmts, runStmts, lazyGlobal = true) {
        super();
        this.uri = uri;
        this.stmts = stmts;
        this.runStmts = runStmts;
        this.lazyGlobal = lazyGlobal;
    }
    toLines() {
        return arrayUtils_1.flatten(this.stmts.map(stmt => stmt.toLines()));
    }
    get start() {
        if (this.stmts.length === 0) {
            return vscode_languageserver_1.Position.create(0, 0);
        }
        return this.stmts[0].start;
    }
    get end() {
        if (this.stmts.length === 0) {
            return vscode_languageserver_1.Position.create(0, 0);
        }
        return this.stmts[this.stmts.length - 1].end;
    }
    get ranges() {
        return [...this.stmts];
    }
    toLocation() {
        return { uri: this.uri, range: { start: this.start, end: this.end } };
    }
    get tag() {
        return types_1.SyntaxKind.script;
    }
}
exports.Script = Script;
//# sourceMappingURL=script.js.map