import { TokenType } from '../entities/tokentypes';
import { Diagnostic } from 'vscode-languageserver';
import { Token } from '../entities/token';
export declare type ITokenMap = Map<string, {
    type: TokenType;
    literal?: any;
}>;
export interface Tokenized {
    tokens: Token[];
    scanErrors: Diagnostic[];
    regions: Token[];
}
export declare const enum ScanKind {
    Whitespace = 0,
    Token = 1,
    Diagnostic = 2,
    Region = 3
}
export declare type Result<T, S extends ScanKind> = {
    result: T;
    kind: S;
};
export declare type TokenResult = Result<Token, ScanKind.Token>;
export declare type WhitespaceResult = Result<null, ScanKind.Whitespace>;
export declare type DiagnosticResult = Result<Diagnostic, ScanKind.Diagnostic>;
export declare type RegionResult = Result<Token, ScanKind.Region>;
export declare type ScanResult = TokenResult | WhitespaceResult | DiagnosticResult | RegionResult;
