"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
if (Symbol['asyncIterator'] === undefined) {
    Symbol['asyncIterator'] = Symbol.for('asyncIterator');
}
const vscode_languageserver_1 = require("vscode-languageserver");
const kls_1 = require("./kls");
const serverUtils_1 = require("./utilities/serverUtils");
const logger_1 = require("./utilities/logger");
const constants_1 = require("./utilities/constants");
// tslint:disable-next-line:import-name
const commander_1 = __importDefault(require("commander"));
const initialize_1 = require("./typeChecker/initialize");
commander_1.default
    .version('0.10.0', '-v --version')
    .option('--node-ipc', 'Connect with node inter process communication')
    .option('--stdio', 'Connect with standard io')
    .option('--clientProcessId', 'Id of the attached client process')
    .option('--harmony_async_iteration', 'Must be enabled if using node.js pre 10.0')
    .parse(process.argv);
// get connection primitives based on command argument
const { reader, writer } = serverUtils_1.getConnectionPrimitives(commander_1.default);
// Create a connection for the server. The connection uses Node's IPC or stdin and stdout
// Also include all preview / proposed LSP features.
exports.connection = vscode_languageserver_1.createConnection(vscode_languageserver_1.ProposedFeatures.all, reader, writer);
// REMOVE ME TODO probably need to refactor the type modules as
// structure and the primitives have a dependency loop
initialize_1.typeInitializer();
// default client configuration
const defaultClientConfiguration = {
    kerbalSpaceProgramPath: undefined,
    telnetHost: '127.0.0.1',
    telnetPort: 5410,
    lspPort: 7000,
    completionCase: 'camelcase',
    trace: {
        server: {
            verbosity: 'off',
            format: 'text',
            level: 'error',
        },
    },
};
// create default server options object
const defaultConfiguration = {
    reader,
    writer,
    workspaceFolder: '',
    workspaceUri: '',
    clientCapability: {
        hasConfiguration: false,
        hasWorkspaceFolder: false,
    },
    keywords: constants_1.keywordCompletions(2 /* camelcase */),
    clientConfig: defaultClientConfiguration,
};
const kls = new kls_1.KLS(2 /* camelcase */, new logger_1.Logger(exports.connection.console, 1 /* info */), exports.connection.tracer, exports.connection, defaultConfiguration);
kls.listen();
//# sourceMappingURL=server.js.map