"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_uri_1 = require("vscode-uri");
const vscode_languageserver_1 = require("vscode-languageserver");
const mockServices_1 = require("./utilities/mockServices");
const analysisService_1 = require("../services/analysisService");
const logger_1 = require("../utilities/logger");
const typeGuards_1 = require("../utilities/typeGuards");
const grandSource = `
runOncePath("parent.ks").
global grandParent is "grandparent".
`;
const greatUncleSource = `
runOncePath("parent.ks").
runOncePath("uncle.ks").
global greatUncle is "great uncle".
`;
const parentSource = `
runOncePath("child.ks").
global parent is "parent".
`;
const uncleSource = `
global uncle is "uncle".
`;
const childSource = `
global child is "child".
`;
describe('Symbol Table', () => {
    test('Global Environment', () => __awaiter(this, void 0, void 0, function* () {
        const grandUri = vscode_uri_1.URI.file('/example/folder/grandParent.ks').toString();
        const greatUncleUri = vscode_uri_1.URI.file('/example/folder/greatUncle.ks').toString();
        const parentUri = vscode_uri_1.URI.file('/example/folder/parent.ks').toString();
        const uncleUri = vscode_uri_1.URI.file('/example/folder/uncle.ks').toString();
        const childUri = vscode_uri_1.URI.file('/example/folder/child.ks').toString();
        const documents = new Map([
            [grandUri, vscode_languageserver_1.TextDocument.create(grandUri, 'kos', 1.0, grandSource)],
            [
                greatUncleUri,
                vscode_languageserver_1.TextDocument.create(greatUncleUri, 'kos', 1.0, greatUncleSource),
            ],
            [parentUri, vscode_languageserver_1.TextDocument.create(parentUri, 'kos', 1.0, parentSource)],
            [uncleUri, vscode_languageserver_1.TextDocument.create(uncleUri, 'kos', 1.0, uncleSource)],
            [childUri, vscode_languageserver_1.TextDocument.create(childUri, 'kos', 1.0, childSource)],
        ]);
        const docService = mockServices_1.createMockDocumentService(documents, vscode_uri_1.URI.file('/').toString());
        const analysisService = new analysisService_1.AnalysisService(2 /* camelcase */, logger_1.mockLogger, logger_1.mockTracer, docService);
        const grandInfo = yield analysisService.getInfo(grandUri);
        const greatUncleInfo = yield analysisService.getInfo(greatUncleUri);
        const parentInfo = yield analysisService.getInfo(parentUri);
        const uncleInfo = yield analysisService.getInfo(uncleUri);
        const childInfo = yield analysisService.getInfo(childUri);
        expect(grandInfo).toBeDefined();
        expect(greatUncleInfo).toBeDefined();
        expect(parentInfo).toBeDefined();
        expect(uncleInfo).toBeDefined();
        expect(childInfo).toBeDefined();
        if (!typeGuards_1.empty(grandInfo) &&
            !typeGuards_1.empty(greatUncleInfo) &&
            !typeGuards_1.empty(parentInfo) &&
            !typeGuards_1.empty(uncleInfo) &&
            !typeGuards_1.empty(childInfo)) {
            expect(grandInfo.symbolTable.globalEnvironment('grandparent', 0 /* dependents */, new Set())).toBeDefined();
            expect(grandInfo.symbolTable.globalEnvironment('greatuncle', 0 /* dependents */, new Set())).toBeUndefined();
            expect(grandInfo.symbolTable.globalEnvironment('parent', 0 /* dependents */, new Set())).toBeDefined();
            expect(grandInfo.symbolTable.globalEnvironment('uncle', 0 /* dependents */, new Set())).toBeUndefined();
            expect(grandInfo.symbolTable.globalEnvironment('child', 0 /* dependents */, new Set())).toBeDefined();
            expect(greatUncleInfo.symbolTable.globalEnvironment('grandparent', 0 /* dependents */, new Set())).toBeUndefined();
            expect(greatUncleInfo.symbolTable.globalEnvironment('greatuncle', 0 /* dependents */, new Set())).toBeDefined();
            expect(greatUncleInfo.symbolTable.globalEnvironment('parent', 0 /* dependents */, new Set())).toBeDefined();
            expect(greatUncleInfo.symbolTable.globalEnvironment('uncle', 0 /* dependents */, new Set())).toBeDefined();
            expect(greatUncleInfo.symbolTable.globalEnvironment('child', 0 /* dependents */, new Set())).toBeDefined();
            expect(parentInfo.symbolTable.globalEnvironment('grandparent', 0 /* dependents */, new Set())).toBeDefined();
            expect(parentInfo.symbolTable.globalEnvironment('greatuncle', 0 /* dependents */, new Set())).toBeDefined();
            expect(parentInfo.symbolTable.globalEnvironment('parent', 0 /* dependents */, new Set())).toBeDefined();
            expect(parentInfo.symbolTable.globalEnvironment('uncle', 0 /* dependents */, new Set())).toBeDefined();
            expect(parentInfo.symbolTable.globalEnvironment('child', 0 /* dependents */, new Set())).toBeDefined();
            expect(uncleInfo.symbolTable.globalEnvironment('grandparent', 0 /* dependents */, new Set())).toBeUndefined();
            expect(uncleInfo.symbolTable.globalEnvironment('greatuncle', 0 /* dependents */, new Set())).toBeDefined();
            expect(uncleInfo.symbolTable.globalEnvironment('parent', 0 /* dependents */, new Set())).toBeDefined();
            expect(uncleInfo.symbolTable.globalEnvironment('uncle', 0 /* dependents */, new Set())).toBeDefined();
            expect(uncleInfo.symbolTable.globalEnvironment('child', 0 /* dependents */, new Set())).toBeDefined();
            expect(childInfo.symbolTable.globalEnvironment('grandparent', 0 /* dependents */, new Set())).toBeDefined();
            expect(childInfo.symbolTable.globalEnvironment('greatuncle', 0 /* dependents */, new Set())).toBeDefined();
            expect(childInfo.symbolTable.globalEnvironment('parent', 0 /* dependents */, new Set())).toBeDefined();
            expect(childInfo.symbolTable.globalEnvironment('uncle', 0 /* dependents */, new Set())).toBeDefined();
            expect(childInfo.symbolTable.globalEnvironment('child', 0 /* dependents */, new Set())).toBeDefined();
        }
    }));
});
//# sourceMappingURL=symbols.spec.js.map