"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const string_1 = require("../typeChecker/ksTypes/primitives/string");
const boolean_1 = require("../typeChecker/ksTypes/primitives/boolean");
const structure_1 = require("../typeChecker/ksTypes/primitives/structure");
const part_1 = require("../typeChecker/ksTypes/parts/part");
const dockingPort_1 = require("../typeChecker/ksTypes/parts/dockingPort");
const typeCreators_1 = require("../typeChecker/typeCreators");
const initialize_1 = require("../typeChecker/initialize");
const userList_1 = require("../typeChecker/ksTypes/collections/userList");
const list_1 = require("../typeChecker/ksTypes/collections/list");
const enumerable_1 = require("../typeChecker/ksTypes/collections/enumerable");
const scalar_1 = require("../typeChecker/ksTypes/primitives/scalar");
const parametricType_1 = require("../typeChecker/parametricTypes/parametricType");
const type_1 = require("../typeChecker/types/type");
initialize_1.typeInitializer();
describe('Type Utilities', () => {
    test('Basic Attributes Parametric Type', () => {
        const parametricType = new parametricType_1.ParametricType('example', { get: true, set: true }, ['T'], 0 /* basic */);
        expect(parametricType.kind).toBe(0 /* basic */);
        expect(parametricType.name).toBe('example');
        expect(parametricType.access.get).toBe(true);
        expect(parametricType.access.set).toBe(true);
        expect(parametricType.getCallSignature()).toBeUndefined();
        expect(parametricType.anyType).toBe(false);
        expect(parametricType.getSuperType()).toBeUndefined();
        expect(parametricType.toString()).toBe('example<T>');
        const genericTypeParameters = parametricType.getTypeParameters();
        expect(genericTypeParameters.length).toBe(1);
        expect(genericTypeParameters[0].name).toBe('T');
        const exampleType = parametricType.apply(string_1.stringType);
        const parameters = exampleType.getTypeParameters();
        expect(exampleType.kind).toBe(0 /* basic */);
        expect(exampleType.name).toBe('example');
        expect(exampleType.access.get).toBe(true);
        expect(exampleType.access.set).toBe(true);
        expect(exampleType.getCallSignature()).toBeUndefined();
        expect(exampleType.getAssignmentType()).toBe(exampleType);
        expect(parameters[0].name).toBe('T');
        expect(exampleType.anyType).toBe(false);
        expect(exampleType.getSuperType()).toBeUndefined();
        expect(exampleType.toString()).toBe('example<string>');
        const typeParameters = exampleType.getTypeParameters();
        expect(typeParameters.length).toBe(1);
        expect(typeParameters[0].name).toBe('T');
    });
    test('Basic Attributes Type', () => {
        const templateType = typeCreators_1.createParametricType('Template', ['T']);
        const type = new type_1.Type('example', { get: true, set: true }, new Map([
            [
                templateType.getTypeParameters()[0],
                new type_1.Type('integer', { get: true, set: true }, new Map(), 0 /* basic */),
            ],
        ]), 0 /* basic */, undefined, templateType, false);
        expect(type.kind).toBe(0 /* basic */);
        expect(type.name).toBe('example');
        expect(type.access.get).toBe(true);
        expect(type.access.set).toBe(true);
        expect(type.getCallSignature()).toBeUndefined();
        expect(type.anyType).toBe(false);
        expect(type.getSuperType()).toBeUndefined();
        expect(type.toString()).toBe('example<integer>');
        const genericTypeParameters = type.getTypeParameters();
        expect(genericTypeParameters.length).toBe(1);
        expect(genericTypeParameters[0].name).toBe('T');
        const exampleType = type.apply(string_1.stringType);
        expect(exampleType).toBe(type);
        expect(type.isSubtypeOf(templateType)).toBe(true);
    });
    test('Is Subtype 1', () => {
        expect(string_1.stringType.isSubtypeOf(string_1.stringType)).toBe(true);
        expect(string_1.stringType.isSubtypeOf(boolean_1.booleanType)).toBe(false);
        expect(string_1.stringType.isSubtypeOf(structure_1.structureType)).toBe(true);
        expect(string_1.stringType.isSubtypeOf(part_1.partType)).toBe(false);
        expect(string_1.stringType.isSubtypeOf(dockingPort_1.dockingPortType)).toBe(false);
        expect(boolean_1.booleanType.isSubtypeOf(string_1.stringType)).toBe(false);
        expect(boolean_1.booleanType.isSubtypeOf(boolean_1.booleanType)).toBe(true);
        expect(boolean_1.booleanType.isSubtypeOf(structure_1.structureType)).toBe(true);
        expect(boolean_1.booleanType.isSubtypeOf(part_1.partType)).toBe(false);
        expect(boolean_1.booleanType.isSubtypeOf(dockingPort_1.dockingPortType)).toBe(false);
        expect(structure_1.structureType.isSubtypeOf(string_1.stringType)).toBe(false);
        expect(structure_1.structureType.isSubtypeOf(boolean_1.booleanType)).toBe(false);
        expect(structure_1.structureType.isSubtypeOf(structure_1.structureType)).toBe(true);
        expect(structure_1.structureType.isSubtypeOf(part_1.partType)).toBe(false);
        expect(structure_1.structureType.isSubtypeOf(dockingPort_1.dockingPortType)).toBe(false);
        expect(part_1.partType.isSubtypeOf(string_1.stringType)).toBe(false);
        expect(part_1.partType.isSubtypeOf(boolean_1.booleanType)).toBe(false);
        expect(part_1.partType.isSubtypeOf(structure_1.structureType)).toBe(true);
        expect(part_1.partType.isSubtypeOf(part_1.partType)).toBe(true);
        expect(part_1.partType.isSubtypeOf(dockingPort_1.dockingPortType)).toBe(false);
        expect(dockingPort_1.dockingPortType.isSubtypeOf(string_1.stringType)).toBe(false);
        expect(dockingPort_1.dockingPortType.isSubtypeOf(boolean_1.booleanType)).toBe(false);
        expect(dockingPort_1.dockingPortType.isSubtypeOf(structure_1.structureType)).toBe(true);
        expect(dockingPort_1.dockingPortType.isSubtypeOf(part_1.partType)).toBe(true);
        expect(dockingPort_1.dockingPortType.isSubtypeOf(dockingPort_1.dockingPortType)).toBe(true);
    });
    test('Is Subtype 2', () => {
        const aType = typeCreators_1.createType('a');
        const bType = typeCreators_1.createType('b');
        const cType = typeCreators_1.createType('c');
        bType.addSuper(typeCreators_1.noMap(aType));
        cType.addSuper(typeCreators_1.noMap(aType));
        expect(bType.isSubtypeOf(aType)).toBe(true);
        expect(cType.isSubtypeOf(aType)).toBe(true);
        expect(aType.isSubtypeOf(bType)).toBe(false);
        expect(cType.isSubtypeOf(bType)).toBe(false);
        expect(bType.isSubtypeOf(cType)).toBe(false);
        expect(aType.isSubtypeOf(cType)).toBe(false);
    });
    test('Has Suffix', () => {
        const aType = typeCreators_1.createType('a');
        const bType = typeCreators_1.createType('b');
        const cType = typeCreators_1.createType('c');
        const dType = typeCreators_1.createType('d');
        aType.addSuffixes(typeCreators_1.noMap(typeCreators_1.createSuffixType('example1', cType)), typeCreators_1.noMap(typeCreators_1.createSuffixType('example2', cType)));
        bType.addSuffixes(typeCreators_1.noMap(typeCreators_1.createSuffixType('example3', cType)));
        bType.addSuper(typeCreators_1.noMap(aType));
        cType.addSuffixes(typeCreators_1.noMap(typeCreators_1.createArgSuffixType('example', dType, dType)));
        expect(aType.getSuffixes().get('example1')).toBeDefined();
        expect(aType.getSuffixes().get('example2')).toBeDefined();
        expect(aType.getSuffixes().get('example3')).toBeUndefined();
        expect(bType.getSuffixes().get('example1')).toBeDefined();
        expect(bType.getSuffixes().get('example2')).toBeDefined();
        expect(bType.getSuffixes().get('example3')).toBeDefined();
        expect(bType.getSuffixes().get('other')).toBeUndefined();
        expect(cType.getSuffixes().get('example')).toBeDefined();
        expect(cType.getSuffixes().get('other')).toBeUndefined();
        expect(dType.getSuffixes().get('any')).toBeUndefined();
    });
    test('Can Coerce', () => {
        expect(structure_1.structureType.canCoerceFrom(scalar_1.scalarType)).toBe(true);
        expect(structure_1.structureType.canCoerceFrom(boolean_1.booleanType)).toBe(true);
        expect(structure_1.structureType.canCoerceFrom(string_1.stringType)).toBe(true);
        expect(structure_1.structureType.canCoerceFrom(enumerable_1.collectionType)).toBe(true);
        expect(structure_1.structureType.canCoerceFrom(list_1.listType)).toBe(true);
        expect(structure_1.structureType.canCoerceFrom(userList_1.userListType)).toBe(true);
        expect(scalar_1.scalarType.canCoerceFrom(structure_1.structureType)).toBe(true);
        expect(scalar_1.scalarType.canCoerceFrom(boolean_1.booleanType)).toBe(false);
        expect(scalar_1.scalarType.canCoerceFrom(string_1.stringType)).toBe(false);
        expect(scalar_1.scalarType.canCoerceFrom(enumerable_1.collectionType)).toBe(false);
        expect(scalar_1.scalarType.canCoerceFrom(list_1.listType)).toBe(false);
        expect(scalar_1.scalarType.canCoerceFrom(userList_1.userListType)).toBe(false);
        expect(boolean_1.booleanType.canCoerceFrom(structure_1.structureType)).toBe(true);
        expect(boolean_1.booleanType.canCoerceFrom(scalar_1.scalarType)).toBe(true);
        expect(boolean_1.booleanType.canCoerceFrom(string_1.stringType)).toBe(true);
        expect(boolean_1.booleanType.canCoerceFrom(enumerable_1.collectionType)).toBe(true);
        expect(boolean_1.booleanType.canCoerceFrom(list_1.listType)).toBe(true);
        expect(boolean_1.booleanType.canCoerceFrom(userList_1.userListType)).toBe(true);
        expect(string_1.stringType.canCoerceFrom(structure_1.structureType)).toBe(true);
        expect(string_1.stringType.canCoerceFrom(scalar_1.scalarType)).toBe(true);
        expect(string_1.stringType.canCoerceFrom(boolean_1.booleanType)).toBe(true);
        expect(string_1.stringType.canCoerceFrom(enumerable_1.collectionType)).toBe(true);
        expect(string_1.stringType.canCoerceFrom(list_1.listType)).toBe(true);
        expect(string_1.stringType.canCoerceFrom(userList_1.userListType)).toBe(true);
        expect(enumerable_1.collectionType.canCoerceFrom(structure_1.structureType)).toBe(true);
        expect(enumerable_1.collectionType.canCoerceFrom(scalar_1.scalarType)).toBe(false);
        expect(enumerable_1.collectionType.canCoerceFrom(boolean_1.booleanType)).toBe(false);
        expect(enumerable_1.collectionType.canCoerceFrom(string_1.stringType)).toBe(false);
        expect(enumerable_1.collectionType.canCoerceFrom(list_1.listType)).toBe(true);
        expect(enumerable_1.collectionType.canCoerceFrom(userList_1.userListType)).toBe(true);
        expect(list_1.listType.canCoerceFrom(structure_1.structureType)).toBe(true);
        expect(list_1.listType.canCoerceFrom(scalar_1.scalarType)).toBe(false);
        expect(list_1.listType.canCoerceFrom(boolean_1.booleanType)).toBe(false);
        expect(list_1.listType.canCoerceFrom(string_1.stringType)).toBe(false);
        expect(list_1.listType.canCoerceFrom(enumerable_1.collectionType)).toBe(false);
        expect(list_1.listType.canCoerceFrom(userList_1.userListType)).toBe(true);
        expect(userList_1.userListType.canCoerceFrom(structure_1.structureType)).toBe(true);
        expect(userList_1.userListType.canCoerceFrom(scalar_1.scalarType)).toBe(false);
        expect(userList_1.userListType.canCoerceFrom(boolean_1.booleanType)).toBe(false);
        expect(userList_1.userListType.canCoerceFrom(string_1.stringType)).toBe(false);
        expect(userList_1.userListType.canCoerceFrom(enumerable_1.collectionType)).toBe(false);
        expect(userList_1.userListType.canCoerceFrom(list_1.listType)).toBe(false);
    });
    test('Type Super', () => {
        const example = new type_1.Type('example', { get: true, set: true }, new Map(), 0 /* basic */);
        example.addSuffixes(typeCreators_1.noMap(typeCreators_1.createSuffixType('suffix1', string_1.stringType)));
        expect(example.getSuperType()).toBeUndefined();
        expect(example.getSuffixes().get('suffix1')).toBeDefined();
        expect(example.getSuffixes().get('suffix2')).toBeUndefined();
        const superExample = new type_1.Type('superExample', { get: true, set: true }, new Map(), 0 /* basic */);
        superExample.addSuffixes(typeCreators_1.noMap(typeCreators_1.createSuffixType('suffix2', string_1.stringType)));
        example.addSuper(typeCreators_1.noMap(superExample));
        expect(example.getSuperType()).toBeDefined();
        expect(example.getSuffixes().get('suffix1')).toBeDefined();
        expect(example.getSuffixes().get('suffix2')).toBeDefined();
        expect(() => example.addSuper(typeCreators_1.noMap(superExample))).toThrow();
    });
    test('Parametric Super', () => {
        const example = new parametricType_1.ParametricType('example', { get: true, set: true }, ['T'], 0 /* basic */);
        example.addSuffixes(typeCreators_1.noMap(typeCreators_1.createSuffixType('suffix1', string_1.stringType)));
        expect(example.getSuperType()).toBeUndefined();
        expect(example.getSuffixes().get('suffix1')).toBeDefined();
        expect(example.getSuffixes().get('suffix2')).toBeUndefined();
        const superExample = new parametricType_1.ParametricType('superExample', { get: true, set: true }, ['T'], 0 /* basic */);
        superExample.addSuffixes(typeCreators_1.noMap(typeCreators_1.createSuffixType('suffix2', string_1.stringType)));
        example.addSuper(typeCreators_1.mapTypes(example, superExample));
        expect(example.getSuperType()).toBeDefined();
        expect(example.getSuffixes().get('suffix1')).toBeDefined();
        expect(example.getSuffixes().get('suffix2')).toBeDefined();
        expect(() => example.addSuper(typeCreators_1.noMap(superExample))).toThrow();
    });
});
//# sourceMappingURL=types.spec.js.map