"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typeGuards_1 = require("../../utilities/typeGuards");
const binder_1 = require("./binder");
const indexer_1 = require("../types/indexer");
/**
 * A class that's responsible for binding type arguments to type
 * parameters of parametric call signatures
 */
class IndexerBinder extends binder_1.Binder {
    /**
     * Construct a new call binder
     * @param names names of the types for substituting
     */
    constructor(names) {
        super(names);
    }
    /**
     * Apply type arguments to this parametric call signature to generate
     * a call signature
     * @param typeArguments the type arguments for this application
     * @param callSignature call signature of indexer
     */
    apply(typeArguments, callSignature) {
        // check validity of the substitution
        this.areValidArguments(typeArguments);
        // check if we've already performed this type replacement
        const cachedTypeSubstitutions = this.getCachedArguments(typeArguments);
        const cache = this.applicationCache.get(cachedTypeSubstitutions);
        if (!typeGuards_1.empty(cache)) {
            return cache;
        }
        const mappedCallReplacement = this.mapArguments(typeArguments, callSignature.mapping);
        const newCallSignature = callSignature.type.apply(mappedCallReplacement);
        // generate the new type
        const newIndexer = new indexer_1.Indexer(newCallSignature, typeArguments);
        // cache application
        this.applicationCache.set(cachedTypeSubstitutions, newIndexer);
        return newIndexer;
    }
}
exports.IndexerBinder = IndexerBinder;
//# sourceMappingURL=indexerBinder.js.map