"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const parametricType_1 = require("./parametricType");
const indexerBinder_1 = require("../binders/indexerBinder");
const typeGuards_1 = require("../../utilities/typeGuards");
class ParametricIndexer extends parametricType_1.ParametricType {
    constructor(typeParameters) {
        super('Indexer', { get: true, set: true }, typeParameters, 3 /* indexer */);
        this.indexBinder = new indexerBinder_1.IndexerBinder(typeParameters);
        this.kind = 3 /* indexer */;
    }
    /**
     * Add parameters to this call signature
     * @param indexMap index to add
     */
    addCallSignature(callSignature) {
        if (callSignature.type.params().length !== 1) {
            throw new Error(`Indexer can only have one parameter provided ${callSignature.type.params().length}`);
        }
        super.addCallSignature(callSignature);
    }
    getTypeParameters() {
        return [...this.indexBinder.typeParameters];
    }
    apply(typeArguments) {
        if (typeGuards_1.empty(this.callSignature)) {
            throw new Error('Indexer call signature must be set');
        }
        if (typeArguments instanceof Map) {
            return this.indexBinder.apply(typeArguments, this.callSignature);
        }
        const typeParameters = this.getTypeParameters();
        if (typeParameters.length !== 1) {
            throw new Error('Must provide a type map if more than one parameter is present.');
        }
        return this.indexBinder.apply(new Map([[typeParameters[0], typeArguments]]), this.callSignature);
    }
}
exports.ParametricIndexer = ParametricIndexer;
//# sourceMappingURL=parametricIndexer.js.map