"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typeGuards_1 = require("../utilities/typeGuards");
const tokentypes_1 = require("../entities/tokentypes");
const boolean_1 = require("./ksTypes/primitives/boolean");
const scalar_1 = require("./ksTypes/primitives/scalar");
const string_1 = require("./ksTypes/primitives/string");
const types_1 = require("./types");
/**
 * This map token types to binary operator kinds
 */
exports.binaryOperatorMap = new Map([
    [tokentypes_1.TokenType.minus, types_1.OperatorKind.subtract],
    [tokentypes_1.TokenType.multi, types_1.OperatorKind.multiply],
    [tokentypes_1.TokenType.div, types_1.OperatorKind.divide],
    [tokentypes_1.TokenType.plus, types_1.OperatorKind.plus],
    [tokentypes_1.TokenType.power, types_1.OperatorKind.power],
    [tokentypes_1.TokenType.less, types_1.OperatorKind.lessThan],
    [tokentypes_1.TokenType.lessEqual, types_1.OperatorKind.lessThanEqual],
    [tokentypes_1.TokenType.greater, types_1.OperatorKind.greaterThan],
    [tokentypes_1.TokenType.greaterEqual, types_1.OperatorKind.greaterThanEqual],
    [tokentypes_1.TokenType.and, types_1.OperatorKind.and],
    [tokentypes_1.TokenType.or, types_1.OperatorKind.or],
    [tokentypes_1.TokenType.equal, types_1.OperatorKind.equal],
    [tokentypes_1.TokenType.notEqual, types_1.OperatorKind.notEqual],
]);
/**
 * This maps tokens types to unary operator kinds
 */
exports.unaryOperatorMap = new Map([
    [tokentypes_1.TokenType.not, types_1.OperatorKind.not],
    [tokentypes_1.TokenType.defined, types_1.OperatorKind.defined],
    [tokentypes_1.TokenType.minus, types_1.OperatorKind.negate],
    [tokentypes_1.TokenType.plus, types_1.OperatorKind.negate],
]);
/**
 * Retrieve the type of the follow token
 * @param token token to retrieve
 */
exports.tokenTrackedType = (token) => {
    // check literals and other tokens
    switch (token.type) {
        case tokentypes_1.TokenType.true:
        case tokentypes_1.TokenType.false:
            return boolean_1.booleanType;
        case tokentypes_1.TokenType.integer:
            return scalar_1.integerType;
        case tokentypes_1.TokenType.double:
            return scalar_1.doubleType;
        case tokentypes_1.TokenType.string:
        case tokentypes_1.TokenType.fileIdentifier:
            return string_1.stringType;
        default:
            // if not a literally we need to lookup tracker
            const { tracker } = token;
            if (typeGuards_1.empty(tracker)) {
                return undefined;
            }
            return tracker.getType({ uri: token.uri, range: token });
    }
};
/**
 * Create a pass type parameter for generics with only one parameter
 * @param type the type to create a pass through
 * @param superType the super type to create a pass through
 */
exports.passThroughTypeParameter = (type, superType) => {
    const superTypeParams = superType.getTypeParameters();
    const typeParams = type.getTypeParameters();
    if (superTypeParams.length !== 1) {
        throw new Error(`Super type ${superType.name} does not have` +
            ` one type parameter but instead has ${superTypeParams.join(', ')}.`);
    }
    if (typeParams.length !== 1) {
        throw new Error(`Type ${superType.name} does not have` +
            ` one type parameter but instead has ${superTypeParams.join(', ')}.`);
    }
    return new Map([[typeParams[0], superTypeParams[0]]]);
};
//# sourceMappingURL=typeUtilities.js.map