"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * This call represents a call signature for a type. This could likely be rolled up into
 * the overarching type classes but allowed some means of separation
 */
class CallSignature {
    /**
     * Construct a new call signature
     * @param paramTypes call parameter types
     * @param returnType call return type
     */
    constructor(paramTypes, returnType) {
        this.name = 'Call Signature';
        this.paramTypes = paramTypes;
        this.returnType = returnType;
    }
    /**
     * Get the parameters of this call signature
     */
    params() {
        return this.paramTypes;
    }
    /**
     * Get the return of this call signature
     */
    returns() {
        return this.returnType;
    }
    /**
     * Get type parameters of this call signature
     */
    getTypeParameters() {
        return [];
    }
    /**
     * Apply type arguments to this call signature
     * @param _ type arguments
     */
    apply(_) {
        return this;
    }
    /**
     * Get a string representation of this call signature
     */
    toString() {
        const paramsStr = this.params()
            .map(p => p.toString())
            .join(', ');
        return `(${paramsStr}) => ${this.returns().toString()}`;
    }
}
exports.CallSignature = CallSignature;
//# sourceMappingURL=callSignature.js.map