"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const typeGuards_1 = require("./typeGuards");
// dummy logger we may need for testing or just performance
exports.mockLogger = {
    level: 0 /* verbose */,
    error: (_) => { },
    warn: (_) => { },
    info: (_) => { },
    log: (_) => { },
    verbose: (_) => { },
};
exports.mockTracer = {
    log: (_, __) => { },
};
exports.consoleLogger = {
    level: 0 /* verbose */,
    error: (message) => console.error(message),
    warn: (message) => console.warn(message),
    info: (message) => console.info(message),
    log: (message) => console.log(message),
    verbose: (message) => console.log(message),
};
exports.consoleTracer = {
    log: (message, verbose) => console.trace(message, verbose),
};
/**
 * Log an exception at the provided level to the provided logger
 * @param logger logger to write to
 * @param err exception to log
 * @param level log level to report
 */
exports.logException = (logger, tracer, err, level) => {
    let method = undefined;
    switch (level) {
        case 4 /* error */:
            method = logger.error.bind(logger);
            break;
        case 3 /* warn */:
            method = logger.warn.bind(logger);
            break;
        case 2 /* log */:
            method = logger.log.bind(logger);
            break;
        case 1 /* info */:
            method = logger.info.bind(logger);
            break;
        case 0 /* verbose */:
            method = logger.verbose.bind(logger);
            break;
        case 5 /* none */:
            return;
        default:
            throw new Error('Unknown log level');
    }
    if (typeof err === 'string') {
        method(err);
        tracer.log(err);
        return;
    }
    if (err instanceof Error) {
        method(`${err.name} ${err.message}`);
        if (!typeGuards_1.empty(err.stack)) {
            method(err.stack);
        }
        tracer.log(err.name);
    }
};
// wrapper class for logger to implement logging levels
class Logger {
    constructor(connection, level) {
        this.connection = connection;
        this.level = level;
    }
    error(message) {
        if (this.level <= 4 /* error */) {
            this.connection.warn(message);
        }
    }
    warn(message) {
        if (this.level <= 3 /* warn */) {
            this.connection.warn(message);
        }
    }
    log(message) {
        if (this.level <= 2 /* log */) {
            this.connection.log(message);
        }
    }
    info(message) {
        if (this.level <= 1 /* info */) {
            this.connection.info(message);
        }
    }
    verbose(message) {
        if (this.level <= 0 /* verbose */) {
            this.connection.info(message);
        }
    }
}
exports.Logger = Logger;
//# sourceMappingURL=logger.js.map