import { KLS } from '../kls';
import { TextDocumentPositionParams, CompletionItem, DocumentSymbolParams, SymbolInformation, MessageReader, MessageWriter, SignatureHelp, Diagnostic } from 'vscode-languageserver';
import { KsSymbolKind } from '../analysis/types';
import { CommanderStatic } from 'commander';
import { ClientConfiguration, DiagnosticUri } from '../types';
import { IParseError } from '../parser/types';
/**
 * The default client configuration if none are available
 */
export declare const defaultClientConfiguration: ClientConfiguration;
/**
 * Get the connection primitives based on the request connection type
 * @param connectionType connection type
 */
export declare const getConnectionPrimitives: (program: CommanderStatic) => {
    writer: MessageWriter;
    reader: MessageReader;
};
/**
 * Map a string to a case kind
 */
export declare const caseMapper: (key: string) => CaseKind;
/**
 * Map a string to a log level
 */
export declare const logMapper: (key: string) => LogLevel;
/**
 * Map a ks symbol kind to a completion kind
 */
export declare const symbolMapper: (key: KsSymbolKind) => 10 | 6 | 3;
/**
 * Get a list of all symbols currently in scope at the given line
 * @param analyzer analyzer instance
 * @param documentPosition the current position in the document
 * @param keywordCompletions a list of keywords to always concat
 */
export declare const symbolCompletionItems: (analyzer: KLS, documentPosition: TextDocumentPositionParams, keywordCompletions: CompletionItem[]) => Promise<CompletionItem[]>;
/**
 * Get a list of all known suffixes give a suffix completion trigger
 * @param analyzer analyzer instance
 * @param documentPosition the current position in the document
 */
export declare const suffixCompletionItems: (analyzer: KLS, documentPosition: TextDocumentPositionParams) => Promise<CompletionItem[]>;
/**
 * Get all symbols in the current document
 * @param analyzer analyzer instance
 * @param documentSymbol document identifier
 */
export declare const documentSymbols: (analyzer: KLS, documentSymbol: DocumentSymbolParams) => Promise<Maybe<SymbolInformation[]>>;
/**
 * The default signature if non can be provided
 */
export declare const defaultSignature: () => SignatureHelp;
/**
 * Convert parser error to diagnostic
 * @param error parser error
 * @param uri uri string
 */
export declare const parseToDiagnostics: (error: IParseError, uri: string) => DiagnosticUri;
/**
 * convert resolver error to diagnostic
 * @param error diagnostic
 * @param uri uri string
 */
export declare const addDiagnosticsUri: (error: Diagnostic, uri: string) => DiagnosticUri;
